@@ -1,6 +1,6 @@
 --[[
 
-File l3build-check.lua Copyright (C) 2018-2021 The LaTeX Project
+File l3build-check.lua Copyright (C) 2018-2022 The LaTeX Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -147,7 +147,7 @@ local function normalize_log(content,engine,errlevels)
     -- Images
     line = gsub(line, "<" .. pattern .. ">", "<../%1>")
     -- luaotfload files start with keywords
-    line = gsub(line, "from " .. pattern .. "%(", "from. ./%1(")
+    line = gsub(line, "from " .. pattern .. "$", "from ../%1")
     line = gsub(line, ": " .. pattern .. "%)", ": ../%1)")
     -- Deal with XeTeX specials
     if match(line, "^%.+\\XeTeX.?.?.?file") then
@@ -246,6 +246,10 @@ local function normalize_log(content,engine,errlevels)
     if match(line, "^%.*\\special%{papersize") then
       return ""
     end
+    -- Remove bidi version in \special lines line
+    if match(line, "BIDI.Fullbanner") then
+      line = gsub(line,"Version %d*%.%d*", "Version ...")
+    end
     -- Remove ConTeXt stuff
     if match(line, "^backend         >") or
        match(line, "^close source    >") or
@@ -750,6 +754,11 @@ function runtest(name, engine, hide, ext, test_type, breakout)
   if match(checkformat,"^context$") then
     function setup(file) return tokens .. ' "' .. file .. '" '  end
   end
+  if match(binary,"make4ht") then
+    function setup(file) return tokens .. ' "' .. file .. '" '  end
+    format = ""
+    checkopts = ""
+  end
   local basename = testdir .. "/" .. name
   local gen_file = basename .. test_type.generated
   local new_file = basename .. "." .. engine .. test_type.generated
