@@ -1,6 +1,6 @@
 --[[
 
-File l3build-typesetting.lua Copyright (C) 2018,2019 The LaTeX3 Project
+File l3build-typesetting.lua Copyright (C) 2018-2021 The LaTeX Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -36,38 +36,25 @@ local match = string.match
 local os_type = os.type
 
 function dvitopdf(name, dir, engine, hide)
-  if match(engine, "^u?ptex$") then
-    run(
-      dir,
-      (forcecheckepoch and setepoch() or "") ..
-     "dvipdfmx  " .. name .. dviext
-       .. (hide and (" > " .. os_null) or "")
-    )
-  else
-    run(
-      dir,
-      (forcecheckepoch and setepoch() or "") ..
-     "dvips " .. name .. dviext
-       .. (hide and (" > " .. os_null) or "")
-       .. os_concat ..
-     "ps2pdf " .. name .. psext
-        .. (hide and (" > " .. os_null) or "")
-    )
-  end
+  run(
+    dir,
+    set_epoch_cmd(epoch, forcecheckepoch) ..
+    "dvips " .. name .. dviext
+      .. (hide and (" > " .. os_null) or "")
+      .. os_concat ..
+    "ps2pdf " .. ps2pdfopt .. name .. psext
+      .. (hide and (" > " .. os_null) or "")
+  )
 end
 
 -- An auxiliary used to set up the environmental variables
 function runcmd(cmd,dir,vars)
-  local dir = dir or "."
-  local dir = abspath(dir)
-  local vars = vars or {}
+  dir = dir or "."
+  dir = abspath(dir)
+  vars = vars or {}
   -- Allow for local texmf files
   local env = os_setenv .. " TEXMFCNF=." .. os_pathsep
-  local localtexmf = ""
-  if texmfdir and texmfdir ~= "" and direxists(texmfdir) then
-    localtexmf = os_pathsep .. abspath(texmfdir) .. "//"
-  end
-  local envpaths = "." .. localtexmf .. os_pathsep
+  local envpaths = "." .. localtexmf() .. os_pathsep
     .. abspath(localdir) .. os_pathsep
     .. dir .. (typesetsearch and os_pathsep or "")
   -- Deal with spaces in paths
@@ -77,7 +64,7 @@ function runcmd(cmd,dir,vars)
   for _,var in pairs(vars) do
     env = env .. os_concat .. os_setenv .. " " .. var .. "=" .. envpaths
   end
-  return run(dir,(forcedocepoch and setepoch() or "") .. env .. os_concat .. cmd)
+  return run(dir,set_epoch_cmd(epoch, forcedocepoch) .. env .. os_concat .. cmd)
 end
 
 function biber(name,dir)
@@ -89,7 +76,7 @@ function biber(name,dir)
 end
 
 function bibtex(name,dir)
-  local dir = dir or "."
+  dir = dir or "."
   if fileexists(dir .. "/" .. name .. ".aux") then
     -- LaTeX always generates an .aux file, so there is a need to
     -- look inside it for a \citation line
@@ -114,7 +101,7 @@ function bibtex(name,dir)
 end
 
 function makeindex(name,dir,inext,outext,logext,style)
-  local dir = dir or "."
+  dir = dir or "."
   if fileexists(dir .. "/" .. name .. inext) then
     if style == "" then style = nil end
     return runcmd(makeindexexe .. " " .. makeindexopts
@@ -127,30 +114,35 @@ function makeindex(name,dir,inext,outext,logext,style)
   return 0
 end
 
-function tex(file,dir)
-  local dir = dir or "."
-  return runcmd(typesetexe .. " " .. typesetopts .. " \"" .. typesetcmds
+function tex(file,dir,cmd)
+  dir = dir or "."
+  cmd = cmd or typesetexe .. " " .. typesetopts
+  return runcmd(cmd .. " \"" .. typesetcmds
     .. "\\input " .. file .. "\"",
     dir,{"TEXINPUTS","LUAINPUTS"})
 end
 
 local function typesetpdf(file,dir)
-  local dir = dir or "."
+  dir = dir or "."
   local name = jobname(file)
   print("Typesetting " .. name)
-  local errorlevel = typeset(file,dir)
+  local fn = typeset
+  local cmd = typesetexe .. " " .. typesetopts
+  if specialtypesetting and specialtypesetting[file] then
+    fn = specialtypesetting[file].func or fn
+    cmd = specialtypesetting[file].cmd or cmd
+  end
+  local errorlevel = fn(file,dir,cmd)
   if errorlevel ~= 0 then
     print(" ! Compilation failed")
     return errorlevel
   end
-  pdfname = name .. pdfext
-  rm(docfiledir,pdfname)
-  return cp(pdfname,dir,docfiledir)
+  return 0
 end
 
-typeset = typeset or function(file,dir)
+function typeset(file,dir,exe)
   dir = dir or "."
-  local errorlevel = tex(file,dir)
+  local errorlevel = tex(file,dir,exe)
   if errorlevel ~= 0 then
     return errorlevel
   end
@@ -163,20 +155,25 @@ typeset = typeset or function(file,dir)
     errorlevel =
       makeindex(name,dir,".glo",".gls",".glg",glossarystyle) +
       makeindex(name,dir,".idx",".ind",".ilg",indexstyle)    +
-      tex(file,dir)
+      tex(file,dir,exe)
     if errorlevel ~= 0 then break end
   end
   return errorlevel
 end
 
 -- A hook to allow additional typesetting of demos
-typeset_demo_tasks = typeset_demo_tasks or function()
+function typeset_demo_tasks()
   return 0
 end
 
 local function docinit()
   -- Set up
+  dep_install(typesetdeps)
+  unpack({sourcefiles, typesetsourcefiles}, {sourcefiledir, docfiledir})
   cleandir(typesetdir)
+  for _,file in pairs(typesetfiles) do
+    cp(file, unpackdir, typesetdir)
+  end
   for _,filetype in pairs(
       {bibfiles, docfiles, typesetfiles, typesetdemofiles}
     ) do
@@ -190,8 +187,6 @@ local function docinit()
   for _,file in pairs(typesetsuppfiles) do
     cp(file, supportdir, typesetdir)
   end
-  depinstall(typesetdeps)
-  unpack({sourcefiles, typesetsourcefiles}, {sourcefiledir, docfiledir})
   -- Main loop for doc creation
   local errorlevel = typeset_demo_tasks()
   if errorlevel ~= 0 then
@@ -200,7 +195,7 @@ local function docinit()
   return docinit_hook()
 end
 
-docinit_hook = docinit_hook or function() return 0 end
+function docinit_hook() return 0 end
 
 -- Typeset all required documents
 -- Uses a set of dedicated auxiliaries that need to be available to others
@@ -210,30 +205,33 @@ function doc(files)
   local done = {}
   for _,typesetfiles in ipairs({typesetdemofiles,typesetfiles}) do
     for _,glob in pairs(typesetfiles) do
-      for _,dir in ipairs({typesetdir,unpackdir}) do
-        for _,file in pairs(tree(dir,glob)) do
-          local path,srcname = splitpath(file)
-          local name = jobname(srcname)
-          if not done[name] then
-            local typeset = true
-            -- Allow for command line selection of files
-            if files and next(files) then
-              typeset = false
-              for _,file in pairs(files) do
-                if name == file then
-                  typeset = true
-                  break
-                end
+      local destpath,globstub = splitpath(glob)
+      destpath = docfiledir .. gsub(gsub(destpath,"^./",""),"^.","")
+      for _,p in ipairs(tree(typesetdir,globstub)) do
+        local path,srcname = splitpath(p.cwd)
+        local name = jobname(srcname)
+        if not done[name] then
+          local typeset = true
+          -- Allow for command line selection of files
+          if files and next(files) then
+            typeset = false
+            for _,file in pairs(files) do
+              if name == file then
+                typeset = true
+                break
               end
             end
-            -- Now know if we should typeset this source
-            if typeset then
-              local errorlevel = typesetpdf(srcname,path)
-              if errorlevel ~= 0 then
-                return errorlevel
-              else
-                done[name] = true
-              end
+          end
+          -- Now know if we should typeset this source
+          if typeset then
+            errorlevel = typesetpdf(srcname,path)
+            if errorlevel ~= 0 then
+              return errorlevel
+            else
+              done[name] = true
+              local pdfname = jobname(srcname) .. pdfext
+              rm(pdfname,destpath)
+              cp(pdfname,path,destpath)
             end
           end
         end
@@ -242,4 +240,3 @@ function doc(files)
   end
   return 0
 end
-
