@@ -245,8 +245,12 @@ end
 function shell(s)
   local h = assert(popen(s, 'r'))
   local t = assert(h:read('*a'))
-  h:close()
-  return t
+  local success = h:close()
+  if (success) then
+   return t
+  else
+   error("\nError from shell command:\n" .. s .. "\n" .. t .. "\n")
+  end
 end
 
 function construct_ctan_post(uploadfile,debug)
@@ -324,7 +328,7 @@ function ctan_single_field(fname,fvalue,max,desc,mandatory)
       vs = vs:gsub('`','\\`')
       vs = vs:gsub('\n','\\n')
 -- for strings on commandline version      ctan_post=ctan_post .. ' --form "' .. fname .. "=" .. vs .. '"'
-      ctan_post=ctan_post .. '\nform="' .. fname .. '=' .. vs .. '"'
+      ctan_post=ctan_post .. '\nform-string="' .. fname .. '=' .. vs .. '"'
     end
   else
     error("The value of the field '" .. fname .."' must be a scalar not a table")
