@@ -2,7 +2,7 @@
 
 --[[
 
-File l3build.lua Copyright (C) 2014-2021 The LaTeX Project
+File l3build.lua Copyright (C) 2014-2022 The LaTeX Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -25,7 +25,7 @@ for those people who are interested.
 --]]
 
 -- Version information
-release_date = "2021-12-14"
+release_date = "2022-04-12"
 
 -- File operations are aided by the LuaFileSystem module
 local lfs = require("lfs")
@@ -134,7 +134,7 @@ else
 end
 
 if #checkconfigs > 1 then
-  if options["target"] == "check" then
+  if options["target"] == "check" or options["target"] == "bundlecheck" then
     local errorlevel = 0
     local failed = { }
     for i = 1, #checkconfigs do
